==> fmrdsnotify_0.1-1.tar.gz: fmrdsnotify-0.1/src/CDesktop.cpp <==
/****************************************************************************
**
** Copyright (C) 2007-2007 Trolltech ASA. All rights reserved.
**
** This file is part of the Graphics Dojo project on Trolltech Labs.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
#include "CDesktop.h"
#include "CDlgSetup.h"
#include <QSvgRenderer>
#include <QPainter>
#include <QPaintEvent>
#include <QPen>
#include <QBrush>

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <QtGui/QX11Info>
#include <QtCore/QByteArray>
#include <QtCore/QCoreApplication>

#include <QtGui/QMessageBox>
#include <QLabel>

CDesktop::CDesktop()
    : QWidget(0, Qt::FramelessWindowHint),
      dirty(true)
{
    //To optimize painting a bit you can set the following:
    //   note that child widgets might not be composed
    //   correctly then though
    //setAttribute(Qt::WA_PaintOnScreen);
    //setAttribute(Qt::WA_NoSystemBackground);
   
    QCoreApplication::instance()->setApplicationName("FM Rds Notify");

    // Get required atoms
    Atom winTypeAtom = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE", false);
    Atom homeAppletAto = XInternAtom(QX11Info::display(), "_HILDON_WM_WINDOW_TYPE_HOME_APPLET", false);
    Atom appletIDAtom = XInternAtom(QX11Info::display(), "_HILDON_APPLET_ID", false);
    Atom utf8Atom = XInternAtom(QX11Info::display(), "UTF8_STRING", false); 
    Atom appletSettingAtom = XInternAtom(QX11Info::display(), "_HILDON_APPLET_SETTINGS", false);
   
    // Set correct window type
    XChangeProperty(QX11Info::display(), winId(), winTypeAtom, XA_ATOM, 32,
		    PropModeReplace, (unsigned char *) &homeAppletAto, 1);

    // Use application name to fill AppletID
    QByteArray id (QCoreApplication::instance()->applicationName().remove(' ').toUtf8());
    XChangeProperty(QX11Info::display(), winId(), appletIDAtom, utf8Atom, 8, 
    	            PropModeReplace, (unsigned char *)id.constData(), id.length());

    // Add setting button. This button is shown when hildon-desktop is in edit mode. 
    int settings = 0;
    XChangeProperty(QX11Info::display(), winId(), appletSettingAtom, XA_CARDINAL, 32,
		    PropModeReplace, (unsigned char*)&settings, 1);
    
    renderer = new QSvgRenderer(QLatin1String(":icons/rds.svg"), this);

    // Mafw Meta data connection
    rdsMafw = new CMafwMeta(this);

    // Mafw Meta data connection
    rdsRTCom = new CRTCom(this);

    // Rds Broadcast writer 
    rdsBroadcast = new CRdsBroadcast(this);
    
    // Connect signal changed
    connect(rdsMafw, SIGNAL(sigChanged(QString,int)), rdsBroadcast, SLOT(slotSetText(QString,int)));
    connect(rdsRTCom, SIGNAL(sigChanged(QString,int)), rdsBroadcast, SLOT(slotSetText(QString,int)));
}


void CDesktop::paintEvent(QPaintEvent *e)
{
    QPainter p(this);
    p.setRenderHint(QPainter::Antialiasing);
    p.setClipRect(e->rect());

    //make sure you clean your widget with a transparent
    //  color before doing any rendering
    //  note the usage of a composition mode Source
    //  it's important!
    p.save();
    p.setCompositionMode(QPainter::CompositionMode_Source);
    p.fillRect(rect(), Qt::transparent);
    p.restore();
    
    if (dirty) {
        cache.fill(Qt::transparent);
        QPainter p(&cache);
        p.setRenderHint(QPainter::Antialiasing);
        renderer->render(&p);
        drawDesktop(&p);
        p.end();
        dirty = false;
    }
    p.drawPixmap(0, 0, cache);
        
}

//Filter show setting requests
bool QWidget::x11Event ( XEvent * event )
{
  static Atom appletShowSettingAtom = XInternAtom(QX11Info::display(), "_HILDON_APPLET_SHOW_SETTINGS", false);

  if (event->xclient.message_type == appletShowSettingAtom ) { 
      CDlgSetup *mySetup = new CDlgSetup();
      mySetup->exec();
      return true;
  }
  return false;
}

void CDesktop::resizeEvent(QResizeEvent *e)
{
    if (e->size() != cache.size()) {
        cache = QPixmap(e->size());
        dirty = true;
        int w = e->size().width();
        int h = e->size().height();
	Q_UNUSED(w);
	Q_UNUSED(h);
    }
}

void CDesktop::drawStatus(QPainter *p, QRectF rect, bool status)
{
    if (status) {
       p->setBrush(Qt::green);
       p->setPen(Qt::green); 
       p->drawRect(rect );
    } else {
       p->setBrush(Qt::red);
       p->setPen(Qt::red); 
       p->drawRect(rect );
    }
    p->setBrush(Qt::NoBrush);
    p->setPen(Qt::white); 
}

void CDesktop::drawDesktop(QPainter *p)
{
    QMatrix mp3Matrix = renderer->matrixForElement("MP3_STATUS");
    mp3Matrix = mp3Matrix.scale(1.245, 1.329);
    QRectF mp3Status = renderer->boundsOnElement("MP3_STATUS");
    drawStatus(p, mp3Matrix.mapRect(mp3Status) , CMafwMeta::self().getEnable() );
    

    QMatrix rtcomMatrix = renderer->matrixForElement("RTCOM_STATUS");
    rtcomMatrix = rtcomMatrix.scale(1.245, 1.338);
    QRectF rtcomStatus = renderer->boundsOnElement("RTCOM_STATUS");
    drawStatus(p, rtcomMatrix.mapRect(rtcomStatus) , CRTCom::self().getEnable() );
    
    QMatrix smsMatrix = renderer->matrixForElement("SMS_STATUS");
    smsMatrix = smsMatrix.scale(1.248, 1.344);
    QRectF smsStatus = renderer->boundsOnElement("SMS_STATUS");
    drawStatus(p, smsMatrix.mapRect(smsStatus) , false );

    QFont localFont = p->font();
    localFont.setPointSize(11);
    p->setFont(localFont);

    mp3Matrix = renderer->matrixForElement("MP3");
    mp3Matrix = mp3Matrix.scale(1.245, 1.296);
    QRectF mp3Rect = renderer->boundsOnElement("MP3");
    p->drawText(mp3Matrix.mapRect(mp3Rect) , Qt::AlignLeft, tr("Musics tags") ); 
    
    rtcomMatrix = renderer->matrixForElement("RTCOM");
    rtcomMatrix = rtcomMatrix.scale(1.245, 1.296);
    QRectF rtcomRect = renderer->boundsOnElement("RTCOM");
    p->drawText(rtcomMatrix.mapRect(rtcomRect) , Qt::AlignLeft, tr("Call Status") ); 
    
    smsMatrix = renderer->matrixForElement("SMS");
    smsMatrix = smsMatrix.scale(1.245, 1.296);
    QRectF smsRect = renderer->boundsOnElement("SMS");
    p->drawText(smsMatrix.mapRect(smsRect), Qt::AlignLeft, tr("SMS Message") ); 

    localFont = p->font();
    localFont.setPointSize(16);
    localFont.setBold(true);
    p->setFont(localFont);
    
    QMatrix titleMatrix = renderer->matrixForElement("TITLE");
    titleMatrix = titleMatrix.scale(1.245, 1.296);
    QRectF titleRect = renderer->boundsOnElement("TITLE");
    p->drawText(titleMatrix.mapRect(titleRect) , Qt::AlignLeft, CRdsBroadcast::self().getDefText() ); 
}

==> fmrdsnotify_0.1-3.tar.gz: fmrdsnotify-0.1/src/CDesktop.cpp <==
/****************************************************************************
**
** Copyright (C) 2007-2007 Trolltech ASA. All rights reserved.
**
** This file is part of the Graphics Dojo project on Trolltech Labs.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
#include "CDesktop.h"
#include "CDlgSetup.h"
#include <QSvgRenderer>
#include <QPainter>
#include <QPaintEvent>
#include <QPen>
#include <QBrush>

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <QtGui/QX11Info>
#include <QtCore/QByteArray>
#include <QtCore/QCoreApplication>

#include <QtGui/QMessageBox>
#include <QLabel>

bool          dirty;

CDesktop::CDesktop()
    : QWidget(0, Qt::FramelessWindowHint),
      state(0)
{
    //To optimize painting a bit you can set the following:
    //   note that child widgets might not be composed
    //   correctly then though
    //setAttribute(Qt::WA_PaintOnScreen);
    //setAttribute(Qt::WA_NoSystemBackground);
    dirty = true;
   
    QCoreApplication::instance()->setApplicationName("FM Rds Notify");

    // Get required atoms
    Atom winTypeAtom = XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE", false);
    Atom homeAppletAto = XInternAtom(QX11Info::display(), "_HILDON_WM_WINDOW_TYPE_HOME_APPLET", false);
    Atom appletIDAtom = XInternAtom(QX11Info::display(), "_HILDON_APPLET_ID", false);
    Atom utf8Atom = XInternAtom(QX11Info::display(), "UTF8_STRING", false); 
    Atom appletSettingAtom = XInternAtom(QX11Info::display(), "_HILDON_APPLET_SETTINGS", false);
   
    // Set correct window type
    XChangeProperty(QX11Info::display(), winId(), winTypeAtom, XA_ATOM, 32,
		    PropModeReplace, (unsigned char *) &homeAppletAto, 1);

    // Use application name to fill AppletID
    QByteArray id (QCoreApplication::instance()->applicationName().remove(' ').toUtf8());
    XChangeProperty(QX11Info::display(), winId(), appletIDAtom, utf8Atom, 8, 
    	            PropModeReplace, (unsigned char *)id.constData(), id.length());

    // Add setting button. This button is shown when hildon-desktop is in edit mode. 
    int settings = 0;
    XChangeProperty(QX11Info::display(), winId(), appletSettingAtom, XA_CARDINAL, 32,
		    PropModeReplace, (unsigned char*)&settings, 1);
    
    renderer = new QSvgRenderer(QLatin1String(":icons/rds.svg"), this);

    // Mafw Meta data connection
    rdsMafw = new CMafwMeta(this);

    // Mafw Meta data connection
    rdsRTCom = new CRTCom(this);

    // Rds Broadcast writer 
    rdsBroadcast = new CRdsBroadcast(this);

    // Connect signal changed
    connect(rdsMafw, SIGNAL(sigChanged(QString,int)), rdsBroadcast, SLOT(slotSetText(QString,int)));
    connect(rdsRTCom, SIGNAL(sigChanged(QString,int)), rdsBroadcast, SLOT(slotSetText(QString,int)));

    QDBusConnection::systemBus().connect( 0, FMTX_PATH, FMTX_IF,  "info", this, SLOT(slotFMTXChanged(QString,QString)));
    
}


void CDesktop::paintEvent(QPaintEvent *e)
{
    QPainter p(this);
    p.setRenderHint(QPainter::Antialiasing);
    p.setClipRect(e->rect());

    //make sure you clean your widget with a transparent
    //  color before doing any rendering
    //  note the usage of a composition mode Source
    //  it's important!
    p.save();
    p.setCompositionMode(QPainter::CompositionMode_Source);
    p.fillRect(rect(), Qt::transparent);
    p.restore();
    
    if (dirty) {
        cache.fill(Qt::transparent);
        QPainter p(&cache);
        p.setRenderHint(QPainter::Antialiasing);
        renderer->render(&p);
    	drawDesktop(&p);
        p.end();
        dirty = false;
    }
    p.drawPixmap(0, 0, cache);
        
}

//Filter show setting requests
bool QWidget::x11Event ( XEvent * event )
{
  static Atom appletShowSettingAtom = XInternAtom(QX11Info::display(), "_HILDON_APPLET_SHOW_SETTINGS", false);

  if (event->xclient.message_type == appletShowSettingAtom ) { 
      CDlgSetup *mySetup = new CDlgSetup();
      mySetup->exec();
      dirty = true;
      update();
      return true;
  }
  return false;
}

void CDesktop::resizeEvent(QResizeEvent *e)
{
    if (e->size() != cache.size()) {
        cache = QPixmap(e->size());
        dirty = true;
        int w = e->size().width();
        int h = e->size().height();
	Q_UNUSED(w);
	Q_UNUSED(h);
    }
}

void CDesktop::drawStatus(QPainter *p, QRectF rect, bool status)
{
    p->save();
    QFont localFont = p->font();
    localFont.setPointSize(12);
    p->setFont(localFont);
    if (status) {
       //p->setBrush(Qt::green);
       p->setPen(Qt::green); 
       p->drawText(rect , Qt::AlignCenter, tr("yes") );
    } else {
       //p->setBrush(Qt::ref);
       p->setPen(Qt::red); 
       p->drawText(rect, Qt::AlignCenter, tr("no") );
    }
    p->restore();
}

void CDesktop::drawDesktop(QPainter *p)
{
    QMatrix mp3Matrix = renderer->matrixForElement("MP3_STATUS");
    mp3Matrix = mp3Matrix.scale(1.245, 1.329);
    QRectF mp3Status = renderer->boundsOnElement("MP3_STATUS");
    if ( state)
    	drawStatus(p, mp3Matrix.mapRect(mp3Status) , CMafwMeta::self().getEnable() );
    

    QMatrix rtcomMatrix = renderer->matrixForElement("RTCOM_STATUS");
    rtcomMatrix = rtcomMatrix.scale(1.245, 1.338);
    QRectF rtcomStatus = renderer->boundsOnElement("RTCOM_STATUS");
    if ( state)
    	drawStatus(p, rtcomMatrix.mapRect(rtcomStatus) , CRTCom::self().getEnable() );
    
    QMatrix smsMatrix = renderer->matrixForElement("SMS_STATUS");
    smsMatrix = smsMatrix.scale(1.248, 1.344);
    QRectF smsStatus = renderer->boundsOnElement("SMS_STATUS");
    if ( state)
        drawStatus(p, smsMatrix.mapRect(smsStatus) , false );

    QFont localFont = p->font();
    localFont.setPointSize(14);
    p->setFont(localFont);
    p->setPen(Qt::white);

    mp3Matrix = renderer->matrixForElement("MP3");
    mp3Matrix = mp3Matrix.scale(1.245, 1.329);
    QRectF mp3Rect = renderer->boundsOnElement("MP3");
    p->drawText(mp3Matrix.mapRect(mp3Rect) , Qt::AlignLeft, tr("Musics tags") ); 
    
    rtcomMatrix = renderer->matrixForElement("RTCOM");
    rtcomMatrix = rtcomMatrix.scale(1.245, 1.338);
    QRectF rtcomRect = renderer->boundsOnElement("RTCOM");
    p->drawText(rtcomMatrix.mapRect(rtcomRect) , Qt::AlignLeft, tr("Call Status") ); 
    
    smsMatrix = renderer->matrixForElement("SMS");
    smsMatrix = smsMatrix.scale(1.245, 1.344);
    QRectF smsRect = renderer->boundsOnElement("SMS");
    p->drawText(smsMatrix.mapRect(smsRect), Qt::AlignLeft, tr("SMS Message") ); 

    localFont = p->font();
    localFont.setPointSize(16);
    localFont.setBold(true);
    p->setFont(localFont);
    
    QMatrix titleMatrix = renderer->matrixForElement("TITLE");
    titleMatrix = titleMatrix.scale(1.245, 1.296);
    QRectF titleRect = renderer->boundsOnElement("TITLE");
    if ( state)
    	p->drawText(titleMatrix.mapRect(titleRect) , Qt::AlignLeft, CRdsBroadcast::self().getDefText() ); 
    else
    	p->drawText(titleMatrix.mapRect(titleRect) , Qt::AlignLeft, tr("Transmiter off") ); 
}

void CDesktop::slotFMTXChanged(QString type, QString status)
{
    if (type == "connected") {
	rdsMafw->setConnect(status.toInt() );
	rdsRTCom->setConnect(status.toInt() );
	dirty = true;
	state = status.toInt();
	update();
    }
}
